#include <libwccl/ops/functions/tset/catfilter.h>
#include <libwccl/ops/functions/constant.h>

namespace Wccl {

std::string CatFilter::to_string(const Corpus2::Tagset& tagset) const
{
	std::ostringstream os;
	os << name(tagset) << "("
		<< pos_expr_->to_string(tagset) << ", "
		<< tagset.get_attribute_name(selector_.get_values()) << ", "
		<< tagset.get_attribute_name(mask_.get_values()) << ")";
	return os.str();
}

std::ostream& CatFilter::write_to(std::ostream& os) const
{
	return os << raw_name() << "("
			<< *pos_expr_ << ", "
			<< selector_.raw_dump() << ", "
			<< mask_.raw_dump() << ")";
}

CatFilter::BaseRetValPtr CatFilter::apply_internal(const FunExecContext& context) const
{
	const boost::shared_ptr<const Position>& pos = pos_expr_->apply(context);
	const SentenceContext& sc = context.sentence_context();
	if (pos->is_outside(sc)) {
		return detail::DefaultFunction<TSet>()->apply(context);
	}

	boost::shared_ptr<TSet> tset = boost::make_shared<TSet>();
	const Corpus2::Token* token = sc.at(*pos);
	foreach (const Corpus2::Lexeme& lexeme, token->lexemes()) {
		if (!lexeme.tag().get_masked(selector_).is_null()) {
			tset->combine_with(lexeme.tag());
		}
	}
	tset->tag_ref().mask_with(mask_);
	return tset;
}

} /* end ns Wccl */
